// 
// Project: Anvil
// Personal Finance Application for Linux
//
// Joseph M. Robertson, jmr, jmrobert5@mchsi.com
//
// Copyright(c) 2001, Joseph M. Robertson.
// This code protected by the GNU Public License.
// See the LICENSE file or http://www.gnu.org for details.
//

#ifndef ACCOUNT_H
#define ACCOUNT_H

#include <string>
#include <vector>
#include "transaction.h"
#include "trbasic.h"
#include "qif4.h"
#include "qif4basic.h"
#include "msg.h"

//robodoc code
//****h* Anvil/Account
//
// NAME
//  Account header
// USAGE
//  #include "account.h"
//  Account a = new Account();
// AUTHOR
//  Joseph M. Robertson, jmr, jmrobert5@home.com
// COPYRIGHT
// (c) 2001 by Joseph M. Robertson
// CREATION DATE
//  11/18/99
// HISTORY
//***
//

class Account{
public:
     Account();
     ~Account();
     //the user defined name for the account
     //if this name not in db then its create table time
     //otherwise load table of transactionsx
     string Name();
     void Name(string name);
     //the state of the account: open, closed, inactive
     //closed accounts don't apply in reports
     //inactive accounts show in reports, but not in everyday lists
     string State();
     void State(string state);
     //the type of the account: bank, loan, investment, credit, etc.
     //this tells you the transaction interface
     string Type();
     void Type(string type);
     //
     float Balance();
     int Size();
     //qif file
     string File();
     //void File(string file);
     //load the file into the local datastruct
     //void Read();
     void Read(string file);
     void Write(string file);

     //add a trans to the vector
     void Add(Transaction* value);
     //put dta
     void Update(int index, Transaction* trans);
     //delete and void items
     void Delete(int index);
     void Void(int index);
     //retrieve data
     Transaction* Get(int index);


private:
     string name;
     string state;
     string type;
     string filename;
     vector<Transaction*> vtranslist;

     Qif4 *q;

     //internal account methods
     void Rebalance(int index) throw(Msg);
     //set clear flags after reconcile
     void ClearList(long indexarray);

};


#endif



