// 
// Project: Anvil
// Personal Finance Application for Linux
//
// Joseph M. Robertson, jmr, jmrobert5@mchsi.com
//
// Copyright(c) 2001, Joseph M. Robertson.
// This code protected by the GNU Public License.
// See the LICENSE file or http://www.gnu.org for details.
//

#include "qif4basic.h"
#include <assert.h>


//robodoc code
//****c* Anvil/Qif4Basic
//
// NAME
//  Qif4Basic class
// USAGE
//  #include "qif4basic.h"
//  Qif4Basic *q = new Qif4Basic();
// AUTHOR
//  Joseph M. Robertson, jmr, jmrobert5@mchsi.com
// COPYRIGHT
// (c) 2001 by Joseph M. Robertson
// CREATION DATE
//  6/20/01
// HISTORY
// NOTES
//  This class reads all transaction types except investment transactions.
// DERIVED FROM
//  Anvil/Qif4
//***
//

Qif4Basic::Qif4Basic(){
}

Qif4Basic::~Qif4Basic(){
}


void Qif4Basic::Read(string filename, vector<Transaction*> *vTrans){     
     //if type is cash then 
     TrBasic *t;// = new TrBasic();
     Split *s = new Split();
    
     int readme = 0;
     int addsplit = 0;
     float balance= 0;
     char temp[1024];

     //test if filename != ""
     if(filename == ""){
	  //throw an empty filename exception
	  //return vTrans;
	  throw Msg("Error: empty file name value.");
     }
     //open input stream
     ifstream infile(filename.c_str());
     if (infile.bad()) {
	  throw Msg("Error: could not open file. ");
     }
     //read loop
     while (1){

	  //readlines until ^
	  infile.getline(temp, sizeof(temp));
	  if (infile.eof()) break;

	  //populate trans
	  line = temp;	  
	  // parse line, first char is the line type

	  char key = line[0];
	  string value = line.substr(1, line.length()-1);
	  value = Trim(value);

	  assert(key);
	  assert(!value.empty());
	  //cout << key << " : " << value << endl;

	  //a hack to read any kind of qif file.
	  //this will cause the read to ignore any
	  //sections not headed by type::bank.
	  if ( key == '!' && readme == 0){
	       // test key if type:bank input rest of lines
	       if (value.substr(0, 9) == "Type:Bank"){
		    readme = 1;
	       }
	  }

	  if (readme){

	  switch (key){
	  case '!':
	       //account type line, should be first line
	       if (value.substr(0, 9) == "Type:Bank"){
		    vector<Split> vSplit;
		    //create trans object
		    t = new TrBasic();
		    s = new Split();
	       }
	       else {
		    //exit this read section
		    readme = 0;
	       }
	       break;
	  case 'D':
	       //date
	       t->Date(value);
	       break;
	  case 'T':
	       //amount, - for payment
	       t->Amount(atof(value.c_str()));
	       break;
	  case 'C':
	       //cleared status
	       t->Status(value);
	       break;
	  case 'N':
	       //number, check or reference
	       t->Number(atoi(value.c_str()));
	       break;
	  case 'P':
	       //payee/description
	       t->Payee(value);
	       break;
	  case 'M':
	       //memo
	       t->Memo(value);
	       break;
	  case 'A':
	       //address can be 5 or 6 lines
	       t->Address(value);
	       break;
	  case 'L':
	       //category/transfer/class
	       t->Category(value);
	       break;
	  case 'S':
	       //category in split
	       s->Category(value);
	       break;
	  case 'E':
	       //memo in split
	       s->Memo(value);
	       break;
	  case '$':
	       //dollar amount of split
	       s->Amount(atof(value.c_str()));
	       vSplit.push_back(*s);
	       //Split s;
	       addsplit = 1;
	       break;
	  case '^':
	       if (addsplit) {
		    //cout << "add split" << endl;
		    t->VSplit(vSplit);
		    //reset split flag
		    addsplit = 0;
		    //make a new empty split vector
		    vector<Split> vSplit;
	       }
	       //set two final values before adding trans    
	       t->ID(vTrans->size());
	       balance += t->Amount();
	       t->Balance(balance);

	       vTrans->push_back(t);
	       //cout << "added trans" << endl;
	       
	       //if this to get right type
	       t = new TrBasic();
	       //count++;
	       break;
	  default:
	       //ignore lines that don't match
	       break;
	  }//end switch	       
	  }

     }//end readfile
     //cout << "close" << endl;
     infile.close();
}

void Qif4Basic::Write(string filename, vector<Transaction*> *vTrans){

     //TrBasic *t = new TrBasic();
     //Transaction *t;

     //open a file to write to
     ofstream outfile(filename.c_str(), ios::out|ios::ate|ios::ate);
     if (!outfile.bad()){

	  //write the first line
	  //this is the account type
	  outfile << "!Type:Bank" << endl;
	  TrBasic *t = new TrBasic();
	  //write out the transaction data
	  vector<Transaction*>::iterator p = vTrans->begin();
	  while( p != vTrans->end()){
	       t = (TrBasic*)(*p);
	       //string s_date = t->Date();
	       if (!t->Date().empty()) outfile << "D" << t->Date() << endl;
	       outfile << "T" << t->Amount() << endl;
	       if (!t->Status().empty()) outfile << "C" << t->Status() << endl;
	       if (t->Number() != 0) outfile << "N" << t->Number() << endl;
	       if (!t->Payee().empty()) outfile << "P" << t->Payee() << endl;
	       if (!t->Memo().empty()) outfile << "M" << t->Memo() << endl;
	       if (!t->Address().empty()) outfile << "A" << t->Address() << endl;
	       if (!t->Category().empty()) outfile << "C" << t->Category() << endl;
	       
	       vector<Split> vsplit = t->VSplit();
	       //handle splits
	       if (!vsplit.empty()) {
		    //outfile << "D" << t->Date() << endl;
		    vector<Split>::iterator q = vsplit.begin();
		    while(q != vsplit.end()){
			 //Split *s = q;
			 if (!q->Category().empty()) outfile << "S" << q->Category() << endl;
			 if (!q->Memo().empty()) outfile << "M" << q->Memo() << endl;
			 outfile << "$" << q->Amount() << endl;
			 q++;
		    }
	       }

	       outfile << "^" << endl;
	       
	       //don't forget to increment
	       p++;
	       
	  }
     }

}
