/* 
 * Project: Anvil
 * Personal Finance Application for Linux
 *
 * Joseph M. Robertson, jmr, jmrobert5@mchsi.com
 *
 * Copyright(c) 2001, Joseph M. Robertson.
 * This code protected by the GNU Public License.
 * See the LICENSE file or http://www.gnu.org for details.
 */

/* a C code interface file for 
 * using account class objects
 * within the SLang interpreter.
 *
 * REMEMBER SLang always wants pointer arguments
 * failure to remember this will result in segfaults
 */

#ifndef SL_INT_H
#define SL_INT_H

#include "account.h"
#include "msg.h"
#include <string>
#include <vector>
#include <stdio.h>

/*  #ifdef __cplusplus */
/*  extern "C" { */
/*  #endif  */

#include "slang.h"

/* robodoc code */
/****h* Anvil/sl_int
 *
 * NAME
 *  S-Lang interface
 * USAGE
 *  #include "sl_int.h"
 *  if (-1 == SLadd_intrin_fun_table (Acc_Intrinsics, NULL)){
 *	exit (EXIT_FAILURE);
 *  }	
 * AUTHOR
 *  Joseph M. Robertson, jmr, jmrobert5@mchsi.com
 * COPYRIGHT
 * (c) 2001 by Joseph M. Robertson
 * CREATION DATE
 *  3/18/01
 * HISTORY
 *  6/21/01  added acc_get/set_type functions.
 * SEE ALSO
 *  The file sl_main.cpp for an example of how to embed S-Lang
 *  into another application and include Anvil functions.
 ******
 */

/*
  you must one of these creation functions 
  to get a reference to an account.
  the access methods always work on the current
  account
*/
int intrin_acc_new();
int intrin_acc_set(int *index);
int intrin_acc_read(char *file);

int intrin_acc_close(int *index);

/* 
   these methods work on the current account
   use intrin_acc_set(acc_set) to set current
*/
void intrin_acc_write(char *file);
int intrin_acc_size();

char *intrin_acc_get_type();
void intrin_acc_set_type(char *type);

char *intrin_acc_get_state();
void intrin_acc_set_state(char *state);

void intrin_acc_add();
void intrin_acc_delete(int *index);
void intrin_acc_void(int *index);

char *intrin_acc_get_transaction(int *index);
void intrin_acc_set_transaction(char *trans);

char *intrin_acc_get_name();
void intrin_acc_set_name(char *name);

char *intrin_acc_field(int *index, int *field);

/* now add the functions to the table */
/*
     MAKE_INTRINSIC_1("acc_set", intrin_acc_set, SLANG_REF_TYPE,
		      SLANG_REF_TYPE),
*/
static SLang_Intrin_Fun_Type Acc_Intrinsics [] =
{		
     MAKE_INTRINSIC_0("acc_new", intrin_acc_new, SLANG_INT_TYPE),
     MAKE_INTRINSIC_1("acc_set", intrin_acc_set, SLANG_INT_TYPE,
		      SLANG_INT_TYPE),
     MAKE_INTRINSIC_1("acc_close", intrin_acc_close, SLANG_INT_TYPE,
		      SLANG_INT_TYPE),
     MAKE_INTRINSIC_1("acc_read", intrin_acc_read, SLANG_INT_TYPE, 
		      SLANG_STRING_TYPE),
     MAKE_INTRINSIC_1("acc_write", intrin_acc_write, SLANG_VOID_TYPE,
		      SLANG_STRING_TYPE),
     MAKE_INTRINSIC_0("acc_size", intrin_acc_size, 
		      SLANG_INT_TYPE),
     MAKE_INTRINSIC_0("acc_get_type", intrin_acc_get_type, 
		      SLANG_STRING_TYPE),
     MAKE_INTRINSIC_1("acc_set_type", intrin_acc_set_type, SLANG_VOID_TYPE,
		      SLANG_STRING_TYPE),
     MAKE_INTRINSIC_0("acc_get_state", intrin_acc_get_state, 
		      SLANG_STRING_TYPE),
     MAKE_INTRINSIC_1("acc_set_state", intrin_acc_set_state, SLANG_VOID_TYPE,
		      SLANG_STRING_TYPE),
     MAKE_INTRINSIC_0("acc_add", intrin_acc_add, SLANG_VOID_TYPE),
     MAKE_INTRINSIC_1("acc_delete", intrin_acc_delete, SLANG_VOID_TYPE,
		      SLANG_INT_TYPE),
     MAKE_INTRINSIC_1("acc_void", intrin_acc_void, SLANG_VOID_TYPE,
		      SLANG_INT_TYPE),
     MAKE_INTRINSIC_1("acc_get_transaction", intrin_acc_get_transaction, SLANG_STRING_TYPE, 
		      SLANG_INT_TYPE),
     MAKE_INTRINSIC_0("acc_get_name", intrin_acc_get_name, SLANG_STRING_TYPE),
     MAKE_INTRINSIC_1("acc_set_name", intrin_acc_set_name, SLANG_VOID_TYPE,
		      SLANG_STRING_TYPE),
     MAKE_INTRINSIC_2("acc_field", intrin_acc_field, SLANG_STRING_TYPE,
		      SLANG_INT_TYPE, SLANG_INT_TYPE),
     SLANG_END_INTRIN_FUN_TABLE
};

/*  #ifdef __cplusplus */
/*  } */
/*  #endif  */

#endif
