// 
// Project: Anvil
// Personal Finance Application for Linux
//
// Joseph M. Robertson, jmr, jmrobert5@mchsi.com
//
// Copyright(c) 2001, Joseph M. Robertson.
// This code protected by the GNU Public License.
// See the LICENSE file or http://www.gnu.org for details.
//

#include "transaction.h"

//robodoc code
//****c* Anvil/Transaction
//
// NAME
//  Transaction class
// CHILDREN
//  Anvil/TrBasic
// USAGE
//  Base Transaction Class
//  This class is used as a base for the 2 transaction
//  types: basic and investment.
// AUTHOR
//  Joseph M. Robertson, jmr, jmrobert5@mchsi.com
// COPYRIGHT
// (c) 2001 by Joseph M. Robertson
// CREATION DATE
//  3/18/01
// HISTORY
//  6/13/01 changed to be a base class,
//          no longer directly instantiated in accounts or qif4.
//***
//

Transaction::Transaction(){
     id = 0;
     date = "";
     status = "";  
     payee = "";
     memo = "";
}

Transaction::~Transaction(){
     //cout << "freeing trans." << endl;
}

//robodoc code
//****m* Anvil/Transaction::ID
// NAME
//  ID - unique number for this transaction.
// SYNOPSIS
//  long Transaction::ID()
//  void Transaction::ID(long value)
// PURPOSE
//  Use to get/set the number of the transaction.
//***
long Transaction::ID(){
     return id;
}

void Transaction::ID(long value){
     id = value;
}


//robodoc code
//****m* Anvil/Transaction::Date
// NAME
//  Date - the date of the transaction, a string value.
// SYNOPSIS
//  string Transaction::Date()
//  void Transaction::Date(string value)
// PURPOSE
//  Use to get/set the date of the transaction.
//***
string Transaction::Date(){
     return date;
}

void Transaction::Date(string value){
     date = value;
}


//robodoc code
//****m* Anvil/Transaction::Status
// NAME
//  Status
// SYNOPSIS
//  string Transaction::Status()
//  void Transaction::Status(string value)
// PURPOSE
//  Use to get/set the status of the transaction.
//***
string Transaction::Status(){
     return status;
}

void Transaction::Status(string value){
     status = value;
}

//robodoc code
//****m* Anvil/transaction::Clear
// NAME
//  Clear
// SYNOPSIS
//  void Transaction::Clear
// PURPOSE
//  Convenience methid to set the status flag to 'x'.
//***
void Transaction::Clear(){
     status = "x";
}


//robodoc code
//****m* Anvil/Transaction::Payee
// NAME
//  Payee
// SYNOPSIS
//  string Transaction::Payee()
//  void Transaction::Payee(string value)
// PURPOSE
//  Use to get/set the payee member of the transaction.
//***
string Transaction::Payee(){
     return payee;
}

void Transaction::Payee(string value){
     payee = value;
}

//robodoc code
//****m* Anvil/Transaction::Memo
// NAME
//  Memo
// SYNOPSIS
//  string Transaction::Memo
// PURPOSE
//  Use to get/set the memo member of the transaction.
//***
string Transaction::Memo(){
     return memo;
}

void Transaction::Memo(string value){
     memo = value;
}


//robodoc code
//****m* Anvil/Transaction::Print()
// NAME
//  Print
// SYNOPSIS
//  istring Transaction::Print()
// PURPOSE
//  Use to get a formatted string of the transaction.
//  This is a verbose version.
//***
string Transaction::Print(){
     string temp;
     string s_id;
     char num[12];

     //translate numeric values to string objects
     sprintf(num, "%ld", id);
     s_id = num;
     

     //populate return string
     temp.append("----------\n");
     temp.append("ID: " + s_id + "\n");
     temp.append("Date: " + date + "\n");
     temp.append("Status: " + status + "\n");
     
     temp.append("Payee: " + payee + "\n");
     temp.append("Memo: " + memo + "\n");
     
     temp.append("----------\n");

     return temp;
}

//robodoc code
//****m* Anvil/Transaction::Print_Rec()
// NAME
//  Print_Rec
// SYNOPSIS
//  string Transaction::Print_Rec()
// PURPOSE
//  Use to print out a string format of the transaction data.
//  This is a short format, pipe delimited string.
//***
string Transaction::Print_Rec(){
     string temp = "";
     string s_id;
     string delim = "|";
     char num[12];

     //translate numeric values to string objects
     sprintf(num, "%ld", id);
     s_id = num;
     

     //populate return string
     temp.append(s_id + delim);
     temp.append(date + delim);
     
     temp.append(status + delim);
     
     temp.append(payee + delim);
     temp.append(memo + delim);
     
     return temp;
}

//robodoc code
//****m* Anvil/Transaction::Field()
// NAME
//  Field
// SYNOPSIS
//  string Transaction::Field(int index)
// PURPOSE
//  Use to get a piece of member data from a transaction.
//  0 = id         6 = memo
//  1 = date       7 = address (could contain linefeeds)
//  2 = number     8 = category
//  3 = status     9 = split  (pipe delimited string)
//  4 = amount         amount|memo|category...
//  5 = payee      10 = balance
//***
//return a string field value by index
string Transaction::Field(int index){
     string temp = "";
     string s_id;
     char num[12];
     string delim = "|";

     //translate numeric values to string objects
     sprintf(num, "%ld", id);
     s_id = num;
     
     //populate return string
     switch (index) {
     case 0:
	  temp = s_id;
	  break;
     case 1:
	  temp = date;
	  break;
     case 2:
	  //temp = s_number;
	  break;
     case 3:
	  temp = status;
	  break;
     case 4:
	  //temp = s_amount;
	  break;
     case 5:
	  temp = payee;
	  break;
     case 6:
	  temp = memo;
	  break;
     case 7:
	  //temp = address;
	  break;
     case 8:
	  //temp = category;
	  break;
     case 9:
	  //  if (split.size() > 0) {
//  	       //temp.append("%" + delim);
//  	       //create an iterator
//  	       vector<Split>::iterator p = split.begin();
//  	       for (int k=0; k<split.size(); k++){
//  		    //get values from item
//  		    sprintf(num, "%8.2f", p->Amount());
//  		    s_amount = num;
//  		    temp.append(s_amount + delim);
//  		    temp.append(p->Memo() + delim);
//  		    temp.append(p->Category() + delim);
//  		    //increment pointer
//  		    p++;
//  	       }
//  	  }
	  break;
     case 10:
	  //temp = s_balance;
	  break;
     default:
	  break;
     }
     return temp;
}


