// 
// Project: Anvil
// Personal Finance Application for Linux
//
// Joseph M. Robertson, jmr, jmrobert5@mchsi.com
//
// Copyright(c) 2001, Joseph M. Robertson.
// This code protected by the GNU Public License.
// See the LICENSE file or http://www.gnu.org for details.
//

#ifndef TRANSACTION_H
#define TRANSACTION_H

#include <stdio.h>
#include <vector>
#include <string>
#include "split.h"
#include "msg.h"

//robodoc code
//****h* Anvil/Transaction
//
// NAME
//  Transaction class
// USAGE
//  Base Transaction Class
//  This class is used as a base for the 2 transaction
//  types: base and investment.
// AUTHOR
//  Joseph M. Robertson, jmr, jmrobert5@mchsi.com
// COPYRIGHT
// (c) 2001 by Joseph M. Robertson
// CREATION DATE
//  3/18/01
// HISTORY
//  6/13/01 changed to be a base class,
//          no longer directly instantiated in accounts or qif4.
//***
//


class Transaction{
public:
     Transaction();
     virtual ~Transaction();
     long ID();
     void ID(long value);
     string Date();
     void Date(string value);
     
     //return the transaction cleared flag
     string Status();
     void Status(string value);
     //clear the transaction
     void Clear();
    

     string Payee();
     void Payee(string value);
     string Memo();
     void Memo(string value);
    
     //formatted string of transaction
     //useful for test or console
     virtual string Print();  
     virtual string Print_Rec();

     //get field data
     virtual string Field(int index);
     
private:
     long id;
     string date;
     string status;
     string payee;
     string memo;
};

#endif
