/*******************************************************
Copyright (C) 2006-2012 Madhan Kanagavel

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
********************************************************/

#include "splittransactionsdialog.h"
#include "constants.h"
#include "splitdetailsdialog.h"
#include "util.h"
#include <wx/statline.h>
#include "model/Model_Category.h"

/*!
 * SplitTransactionDialog type definition
 */

IMPLEMENT_DYNAMIC_CLASS( SplitTransactionDialog, wxDialog )

/*!
 * SplitTransactionDialog event table definition
 */

 BEGIN_EVENT_TABLE(SplitTransactionDialog, wxDialog)
     EVT_BUTTON(wxID_ADD, SplitTransactionDialog::OnButtonAddClick)
     EVT_BUTTON(wxID_REMOVE, SplitTransactionDialog::OnButtonRemoveClick)
     EVT_BUTTON(wxID_EDIT, SplitTransactionDialog::OnButtonEditClick)
     EVT_BUTTON(wxID_OK, SplitTransactionDialog::OnOk)
     EVT_DATAVIEW_ITEM_ACTIVATED(wxID_ANY, SplitTransactionDialog::OnListDblClick)
     EVT_DATAVIEW_SELECTION_CHANGED(wxID_ANY, SplitTransactionDialog::OnListItemSelected)
     //EVT_DATAVIEW_ITEM_CONTEXT_MENU(wxID_ANY, SplitTransactionDialog::OnItemRightClick)
 END_EVENT_TABLE()

SplitTransactionDialog::SplitTransactionDialog( )
{
}

SplitTransactionDialog::SplitTransactionDialog( wxWindow* parent
    , Model_Splittransaction::Data_Set* splits 
    , int transType
    , int accountID)
    : m_splits(splits)
    , accountID_(accountID)
    , items_changed_(false)
{
    for (const auto &item : *m_splits) m_local_splits.push_back(item);

    transType_ = transType;
    selectedIndex_ = -1;
    if (transType_ == Model_Checking::TRANSFER)
        transType_ = Model_Checking::WITHDRAWAL;

    long style = wxCAPTION | wxRESIZE_BORDER | wxSYSTEM_MENU | wxCLOSE_BOX;
    Create(parent, wxID_ANY, _("Split Transaction Dialog")
        , wxDefaultPosition, wxSize(400, 400), style);
}

bool SplitTransactionDialog::Create(wxWindow* parent
    , wxWindowID id
    , const wxString& caption
    , const wxPoint& pos
    , const wxSize& size
    , long style)
{
    lcSplit_ = nullptr;
    SetExtraStyle(GetExtraStyle()|wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    GetSizer()->Fit(this);
    GetSizer()->SetSizeHints(this);
    Centre();

    DataToControls();

    return TRUE;
}

void SplitTransactionDialog::DataToControls()
{
    Model_Currency::Data *currency = Model_Currency::GetBaseCurrency();
    Model_Account::Data *account = Model_Account::instance().get(accountID_);
    if (account) currency = Model_Account::currency(account);

    lcSplit_->DeleteAllItems();
    for (const auto & entry : this->m_local_splits)
    {
        const Model_Category::Data* category = Model_Category::instance().get(entry.CATEGID);
        const Model_Subcategory::Data* sub_category = (entry.SUBCATEGID != -1 ? Model_Subcategory::instance().get(entry.SUBCATEGID) : 0);

        wxVector<wxVariant> data;
        data.push_back(wxVariant(Model_Category::full_name(category, sub_category)));
        data.push_back(wxVariant(Model_Currency::toString(entry.SPLITTRANSAMOUNT, currency)));
        lcSplit_->AppendItem(data, (wxUIntPtr)entry.SPLITTRANSID);
        if (lcSplit_->GetItemCount()-1 == selectedIndex_) lcSplit_->SelectRow(selectedIndex_);
    }
    UpdateSplitTotal();
    SetDisplayEditDeleteButtons();
    itemButtonNew_->SetFocus();
}

void SplitTransactionDialog::CreateControls()
{
    wxBoxSizer* dialogMainSizerV = new wxBoxSizer(wxVERTICAL);
    this->SetSizer(dialogMainSizerV);

    wxStaticText* headingText = new wxStaticText(this, wxID_STATIC, _(" Split Category Details"));
    dialogMainSizerV->Add(headingText, g_flags);

    wxBoxSizer* listCtrlSizer = new wxBoxSizer(wxHORIZONTAL);
    dialogMainSizerV->Add(listCtrlSizer, g_flagsExpand);

    lcSplit_ = new wxDataViewListCtrl(this, wxID_ANY
        , wxDefaultPosition, wxSize(265, 150));
    lcSplit_->AppendTextColumn(_("Category"), wxDATAVIEW_CELL_INERT, 180);
    lcSplit_->AppendTextColumn(_("Amount"), wxDATAVIEW_CELL_INERT, 180);

    listCtrlSizer->Add(lcSplit_, g_flagsExpand);

    wxBoxSizer* totalAmountSizer = new wxBoxSizer(wxHORIZONTAL);
    wxString totalMessage = _("Total:");
    if (transType_ == Model_Checking::WITHDRAWAL)
    {
        totalMessage.Prepend(" ");
        totalMessage.Prepend(_("Withdrawal"));
    }
    if (transType_ == Model_Checking::DEPOSIT)
    {
        totalMessage.Prepend(" ");
        totalMessage.Prepend(_("Deposit"));
    }
    wxStaticText* transAmountText = new wxStaticText(this, wxID_STATIC, totalMessage);
    transAmount_ = new wxStaticText(this, wxID_STATIC, wxEmptyString);
    totalAmountSizer->Add(transAmountText, g_flags);
    totalAmountSizer->Add(transAmount_, g_flags);
    dialogMainSizerV->Add(totalAmountSizer, g_flags);

    wxPanel* buttons_panel = new wxPanel(this, wxID_ANY);
    dialogMainSizerV->Add(buttons_panel, wxSizerFlags(g_flags).Center().Border(wxALL, 5));
    wxStdDialogButtonSizer*  buttons_sizer = new wxStdDialogButtonSizer;
    buttons_panel->SetSizer(buttons_sizer);

    wxSizerFlags flags = wxSizerFlags(g_flags).Border(wxLEFT | wxRIGHT | wxBOTTOM, 5).Center();
    wxBoxSizer* mainButtonSizer = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer* topRowButtonSizer = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer* bottomRowButtonSizer = new wxBoxSizer(wxHORIZONTAL);
    mainButtonSizer->Add(topRowButtonSizer, flags);
    mainButtonSizer->Add(bottomRowButtonSizer, flags);
    buttons_sizer->Add(mainButtonSizer);

    itemButtonNew_ = new wxButton(buttons_panel, wxID_ADD, _("&Add "));
    itemButtonEdit_ = new wxButton(buttons_panel, wxID_EDIT, _("&Edit "));
    itemButtonDelete_ = new wxButton(buttons_panel, wxID_REMOVE, _("&Remove "));
    topRowButtonSizer->Add(itemButtonNew_, flags);
    topRowButtonSizer->Add(itemButtonEdit_, flags);
    topRowButtonSizer->Add(itemButtonDelete_, flags);

    itemButtonOK_ = new wxButton(buttons_panel, wxID_OK, _("&OK "));
    wxButton* itemButtonCancel = new wxButton(buttons_panel, wxID_CANCEL, _("&Cancel "));
    itemButtonCancel->SetFocus();

    bottomRowButtonSizer->Add(itemButtonOK_, g_flags);
    bottomRowButtonSizer->Add(itemButtonCancel, g_flags);
}

void SplitTransactionDialog::OnButtonAddClick( wxCommandEvent& /*event*/ )
{
    Model_Splittransaction::Data *split = Model_Splittransaction::instance().create();
    SplitDetailDialog sdd(this, split, transType_, accountID_);
    if (sdd.ShowModal() == wxID_OK)
    {
        this->m_local_splits.push_back(*split);
        items_changed_ = true;
    }
    DataToControls();
}

void SplitTransactionDialog::OnButtonEditClick( wxCommandEvent& /*event*/ )
{
    EditEntry(selectedIndex_);
    DataToControls();
}

void SplitTransactionDialog::OnOk( wxCommandEvent& /*event*/ )
{
    //Check total amount - should be positive
    if (Model_Splittransaction::instance().get_total(this->m_local_splits) < 0) 
    {
        mmShowErrorMessage(this, _("Invalid Total Amount"), _("Error"));
    }
    else
    {
        // finally 
        this->m_splits->swap(this->m_local_splits);
        EndModal(wxID_OK);
    }
}

void SplitTransactionDialog::OnButtonRemoveClick( wxCommandEvent& event )
{
    if (selectedIndex_ < 0 || selectedIndex_ >= (int)this->m_local_splits.size())
        return;
    this->m_local_splits.erase(this->m_local_splits.begin() + selectedIndex_);
    selectedIndex_ = -1;
    items_changed_ = true;
    DataToControls();
}

void SplitTransactionDialog::UpdateSplitTotal()
{
    double total = Model_Splittransaction::get_total(this->m_local_splits);
    transAmount_->SetLabel(Model_Currency::toCurrency(total));
}

void SplitTransactionDialog::EditEntry(int index)
{
    if (index < 0 || index >= (int)this->m_local_splits.size()) return;
    Model_Splittransaction::Data& split = this->m_local_splits[index];
    SplitDetailDialog sdd(this, &split, transType_, accountID_);
    if (sdd.ShowModal() == wxID_OK)
    {
        items_changed_ = true;
        DataToControls();
        UpdateSplitTotal();
    }
}

void SplitTransactionDialog::OnListItemSelected(wxDataViewEvent& event)
{
    wxDataViewItem item = event.GetItem();
    selectedIndex_ = lcSplit_->ItemToRow(item);
    SetDisplayEditDeleteButtons();
}

void SplitTransactionDialog::OnListDblClick(wxDataViewEvent& event)
{
    if (itemButtonEdit_->IsShown()) EditEntry(selectedIndex_);
}

void SplitTransactionDialog::SetDisplaySplitCategories()
{
    itemButtonNew_->Hide();
    itemButtonEdit_->Hide();
    itemButtonDelete_->Hide();
    itemButtonOK_->Hide();
}

void SplitTransactionDialog::SetDisplayEditDeleteButtons()
{
    bool active = selectedIndex_ >= 0 && selectedIndex_ < (int)this->m_local_splits.size();
    itemButtonEdit_->Enable(active);
    itemButtonDelete_->Enable(active);
}
